use std::{fs, collections::HashMap};

use serde::Serialize;

use crate::error::{Redacted, redact};

#[derive(Serialize)]
struct Logins {
    auths: HashMap<String, Credentials>
}

#[derive(Serialize)]
struct Credentials {
    auth: String
}

impl Credentials {
    fn new(username: &str, password: &str) -> Credentials {
        let mut s = String::new();
        s.push_str(username);
        s.push_str(":");
        s.push_str(password);
        Credentials {
            auth: base64::encode(s)
        }
    }
}

pub fn login(registry: &str, username: &str, password: &str) -> Result<(), Redacted> {
    let home = std::env::var("HOME").unwrap();
    redact(fs::create_dir_all(format!("{}/.docker", home)), password)?;
    let mut auths = HashMap::new();

    auths.insert(String::from(registry), Credentials::new(username, password));

    let data = Logins {
        auths
    };

    let file = redact(fs::OpenOptions::new()
        .write(true)
        .create(true)
        .open(format!("{}/.docker/config.json", home)), password)?;
    redact(serde_json::to_writer(file, &data), password)
}
