use std::error::Error;
use std::fmt;

pub struct Redacted {
    inner: Box<dyn Error>,
    target: String,
}

pub fn redact<T, E: Error + 'static>(r: Result<T, E>, target: &str) -> Result<T, Redacted> {
    match r {
        Err(e) => Err(Redacted {
            inner: Box::new(e),
            target: String::from(target),
        }),
        Ok(v) => Ok(v)
    }
}

impl Error for Redacted { }

impl Redacted {
    fn format(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        let s = format!("{}", self.inner);
        write!(f, "{}", s.replace(&self.target, "<redacted>"))
    }
}

impl fmt::Display for Redacted {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        self.format(f)
    }
}

impl fmt::Debug for Redacted {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        self.format(f)
    }
}
